# -*- coding: utf-8 -*-
import inspect
import logging
import threading
import time

from importlib import import_module

from django.conf import settings
from django.core import management
from django.db.models import Q
from django.utils import module_loading, timezone


class JobHandler:
    def __init__(self, instance, **kwargs):
        self.parameters = kwargs
        self.instance = instance
        self.logs_cleaning()
        self.logger = logging.getLogger('cron')
        self.logger.info("***************************************")
        self.logger.info('LOG' + " " + self.instance.name + " id: " + str(self.instance.pk))
        self.logger.info('RECEIVED PARAMETERS' + str(self.parameters))
        self.logger.info("***************************************")
        self.elaboration_time = None
        self.elaboration_results = None
        self.is_valid = None
        self.extra_info = None
        self.log_file_content = None

    def do(self):
        start = timezone.now()
        try:
            self.elaboration_results = self.run()
            self.is_valid = self.get_is_valid()
        except Exception as e:
            self.elaboration_results = {
                "error_message": str(e),
                "error_type": e.__class__.__name__,
            }
            self.is_valid = False
        end = timezone.now()
        self.elaboration_time = end - start
        return self

    def logs_cleaning(self):
        """Delete old job logs"""
        now, query = timezone.now(), Q()
        success = settings.CRON_JOBLOG_SUCCESS_DURATION
        fail = settings.CRON_JOBLOG_FAIL_DURATION

        if not success and not fail:
            return

        if success:
            query |= Q(is_valid=True, date_last_modify__lt=now - success)
        if fail:
            query |= Q(is_valid=False, date_last_modify__lt=now - fail)

        self.instance.job_elaborations.filter(query).delete()

    def get_is_valid(self) -> bool:
        """Return True if elaboration results are valid."""
        raise NotImplementedError("get_is_valid() must be implemented")

    def run(self):
        """
        Write Job logic here.
        self.parameters contains the required parameters.
        """
        raise NotImplementedError("run() must be implemented")

    @classmethod
    def create(cls, creator, job_name, job_command, parameters={}):
        from thx.cron.models import Job
        try:
            job = Job.objects.create(
                name=job_name,
                command=job_command,
                is_una_tantum=True,
                creator=creator,
                last_modifier=creator,
                parameters=parameters,
            )
            return True, job
        except Exception as e:
            return False, e


def get_command():
    from django.apps import apps

    commands_list = []
    app_list = apps.get_app_configs()
    for app in app_list:
        try:
            import_module(f'{app.name}.jobs')
            jobs_module = module_loading.import_string(f'{app.name}.jobs')
            for name in dir(jobs_module):
                obj = getattr(jobs_module, name)
                if inspect.isclass(obj) and issubclass(obj, JobHandler) and obj.__name__ != "JobHandler":
                    commands_list.append(app.name + ".jobs." + obj.__name__)
        except ModuleNotFoundError as e:
            pass

    return [(command, command) for command in commands_list]


class AsyncRunJobs(threading.Thread):
    def run(self, sleep: int = 0):
        time.sleep(sleep)
        self._run_jobs()

    @staticmethod
    def _run_jobs():
        management.call_command('run_jobs')
